/*
 * Copyright (C) 2006 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.uc.base.net.adaptor;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Manages received headers
 */
public class Headers {
    
    // initial values.
    public final static int NO_CONN_TYPE = 0;
    public final static long NO_TRANSFER_ENCODING = 0;
    public final static long NO_CONTENT_LENGTH = -1;
    
    public final static String METHOD_POST = "POST";
    public final static String METHOD_GET = "GET";
    public final static String METHOD_CONNECT = "CONNECT";
    public final static String METHOD_HEAD = "HEAD";
    public final static String METHOD_PUT = "PUT";
    public final static String METHOD_DELETE = "DELETE";
    public final static String METHOD_OPTIONS = "OPTIONS";
    public final static String METHOD_TRACE = "TRACE";
    
    public final static String HOST = "Host";
    public final static String PROXY = "proxy";
    
    public final static String ACCEPT_ENCODING = "Accept-Encoding";
    // header strings
    public final static String TRANSFER_ENCODING = "Transfer-Encoding";
    public final static String CONTENT_LENGTH = "Content-Length";
    public final static String CONTENT_TYPE = "Content-Type";
    public final static String CONTENT_ENCODING = "Content-Encoding";
    public final static String CONNECTION = "Connection";
    
    public static final String CONNECTION_CLOSE = "close";
    public static final String CONNECTION_KEEP_ALIVE = "keep-alive";
    
    public final static String LOCATION = "Location";
    public final static String PROXY_CONNECTION = "Proxy-Connection";
    
    public final static String REFERER = "Referer";
    public final static String AUTHORIZATION = "Authorization";
    public final static String COOKIE = "Cookie";
    
    public final static String WWW_AUTHENTICATE = "Www-Authenticate";
    public final static String PROXY_AUTHENTICATE = "Proxy-Authenticate";
    public final static String CONTENT_DISPOSITION = "Content-Disposition";
    public final static String ACCEPT_RANGES = "Accept-Ranges";
    public final static String EXPIRES = "Expires";
    public final static String CACHE_CONTROL = "Cache-Control";
    public final static String LAST_MODIFIED = "Last-Modified";
    public final static String ETAG = "Etag";
    public final static String SET_COOKIE = "Set-Cookie";
    public final static String PRAGMA = "Pragma";
    public final static String REFRESH = "Refresh";
    public final static String X_PERMITTED_CROSS_DOMAIN_POLICIES = "X-Permitted-Cross-Domain-Policies";
    public final static String PRIVATE_ENCODING = "Private-Encoding";
    public final static String PRIVATE_ENCODING_GZ2 = "gz2";

    public final static String ZSTD_DICTID = "Zstd-Dictid";

    public final static String ENCODING_GZIP = "gzip";
    public final static String ENCODING_ZSTD = "zstd";

    protected long mContentLength; // Content length of the incoming data
    protected String mConnectionType;
    
    protected ArrayList<Header> mHeaderList = new ArrayList<Header>();
    
    public Headers() {
        mContentLength = NO_CONTENT_LENGTH;
        mConnectionType = "";
    }

    public void addHeader(Header header){
        mHeaderList.add(header);
    }
    
    public String getTransferEncoding() {
        return getFirstHeader(TRANSFER_ENCODING);
    }
    
    public long getContentLength() {
        String value = getFirstHeader(CONTENT_LENGTH);
        if (value != null) {
            try {
                mContentLength = Long.parseLong(value);
            } catch (NumberFormatException e) {
                //ExceptionHandler.processFatalException(e);
                e.printStackTrace();
            }
        }
        return mContentLength;
    }
    
    public String getConnectionType() {
        return mConnectionType;
    }
    
    public String getContentType() {
        return getFirstHeader(CONTENT_TYPE);
    }
    
    public String getContentEncoding() {
        return getFirstHeader(CONTENT_ENCODING);
    }
    
    public String getLocation() {
        return getFirstHeader(LOCATION);
    }
    
    public String getWwwAuthenticate() {
        return getFirstHeader(WWW_AUTHENTICATE);
    }
    
    public String getProxyAuthenticate() {
        return getFirstHeader(PROXY_AUTHENTICATE);
    }
    
    public String getContentDisposition() {
        return getFirstHeader(CONTENT_DISPOSITION);
    }
    
    public String getAcceptRanges() {
        return getFirstHeader(ACCEPT_RANGES);
    }
    
    public String getExpires() {
        return getFirstHeader(EXPIRES);
    }
    
    public String getCacheControl() {
        return getCondensedHeader(CACHE_CONTROL);
    }
    
    public String getLastModified() {
        return getFirstHeader(LAST_MODIFIED);
    }
    
    public String getEtag() {
        return getFirstHeader(ETAG);
    }
    
    public String[] getCookies() {
        return getHeaders(SET_COOKIE);
    }
    
    public String getPragma() {
        return getFirstHeader(PRAGMA);
    }
    
    public String getRefresh() {
        return getFirstHeader(REFRESH);
    }
    
    public String getXPermittedCrossDomainPolicies() {
        return getFirstHeader(X_PERMITTED_CROSS_DOMAIN_POLICIES);
    }
    
    public void setConnectionType(String type) {
        mConnectionType = type;
    }
    
    /**
     * Gets the first header with the given name.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header to get
     * @return the first header or <code>null</code>
     */
    public String getFirstHeader(String name) {
        if (name == null) {
            return null;
        }
        
        for (int i = 0; i < mHeaderList.size(); i++) {
            Header header = mHeaderList.get(i);
            if (name.equalsIgnoreCase(header.name)) {
                return header.value;
            }
        }
        
        return null;
    }
    
    /**
     * Gets the last header with the given name.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header to get
     * @return the last header or <code>null</code>
     */
    public String getLastHeader(String name) {
        if (name == null) {
            return null;
        }
        
        for (int i = mHeaderList.size() - 1; i >= 0; i--) {
            Header header = mHeaderList.get(i);
            if (name.equalsIgnoreCase(header.name)) {
                return header.value;
            }
        }
        
        return null;
    }
    
    /**
     * Gets a header representing all of the header values with the given name. If more that one header with the given
     * name exists the values will be combined with a "," as per RFC 2616.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header(s) to get
     * @return a header string array with a condensed value or <code>null</code> if no headers by the given name are
     *         present
     */
    public String getCondensedHeader(String name) {
        String[] headers = getHeaders(name);
        
        if (headers.length == 0) {
            return null;
        } else if (headers.length == 1) {
            return headers[0];
        } else {
            StringBuilder builder = new StringBuilder();
            builder.append(headers[0]);
            for (int i = 1; i < headers.length; i++) {
                builder.append(", ");
                builder.append(headers[i]);
            }
            return builder.toString();
        }
    }
    
    /**
     * Gets all of the headers with the given name. The returned array maintains the relative order in which the headers
     * were added.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header(s) to get
     * 
     * @return an array of length >= 0
     */
    public String[] getHeaders(String name) {
        if (name == null) {
            return null;
        }
        
        ArrayList<String> headersFound = new ArrayList<String>();
        
        for (int i = 0; i < mHeaderList.size(); i++) {
            Header header = mHeaderList.get(i);
            if (name.equalsIgnoreCase(header.name)) {
                headersFound.add(header.value);
            }
        }
        
        return headersFound.toArray(new String[headersFound.size()]);
    }
    
    public Header[] getAllHeaders() {
        return mHeaderList.toArray(new Header[mHeaderList.size()]);
    }

    public boolean isGz2Encoding() {
        String privateEncoding = getFirstHeader(PRIVATE_ENCODING);
        return PRIVATE_ENCODING_GZ2.equalsIgnoreCase(privateEncoding);
    }
    
    /* for debug only */
    @Override
    public String toString() {
        Iterator<Header> it = mHeaderList.iterator();
        StringBuffer buffer = new StringBuffer();
        buffer.append("head ---> \n");
        while (it.hasNext()) {
            Header header = it.next();
            buffer.append(header.name + ": " + header.value + " \n");
        }
        return buffer.toString();
    }
    
    public static class Header {
        private String name;
        private String value;
        
        public Header() {
            
        }
        
        public Header(String name, String value) {
            this.name = name;
            this.value = value;
        }
        
        public String getName() {
            return name;
        }
        
        public void setName(String name) {
            this.name = name;
        }
        
        public void setValue(String value) {
            this.value = value;
        }
        
        public String getValue() {
            return value;
        }
    }
    
}
