/**
 * ****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-3-4
 * <p>
 * Description :
 * <p>
 * Creation    : 2014-3-4
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-3-4, linhz, Create the file
 * ****************************************************************************
 */
package com.uc.base.net.unet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.WritableByteChannel;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;

import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;

import com.alibaba.mbg.unet.Request;
import com.alibaba.mbg.unet.RequestException;
import com.alibaba.mbg.unet.Response;

import com.uc.base.net.IDataDecompressEventListener;
import com.uc.base.net.IHttpClientAsync;
import com.uc.base.net.IHttpEventListener;
import com.uc.base.net.IRequest;
import com.uc.base.net.adaptor.Headers;
import com.uc.base.net.metrics.HttpConnectionMetricsType;
import com.uc.base.net.unet.util.HttpUtil;
import com.uc.base.net.unet.util.NetLog;
import com.uc.base.net.util.AppNetworkDelegate;
import com.uc.base.net.util.dns.BackupIpProvider;

public class HttpClientAsync extends HttpClientBase implements IHttpClientAsync {

    private fixLooperForExecutor mExecutor;
    private RequestCallback mRequestCallback;
    private Looper mLooper;
    private int mRequestTimeOut = -1;

    protected IHttpEventListener mEventListener;
    com.uc.base.net.unet.adaptor.UnetRequestAdaptor mRequestAdaptor;
    // 存储需要重试的BackupIp列表
    private String[] mRetryingBackupIps;
    // 下一个BackupIp索引
    private int mNextBackupIpRetryIndex;
    private Request mOriginRequest;

    //cause unet use Executor rather than looper.
    class fixLooperForExecutor implements Executor {
        private Handler mHandler;//= new Handler(Looper.getMainLooper());

        public fixLooperForExecutor(Looper looper) {
            mHandler = new Handler(looper);
        }

        @Override
        public void execute(Runnable command) {
            mHandler.post(command);
        }
    }

    /**
     * construct a new object to get messages from Internet asynchronous
     *
     * @param listener
     *            call back function. to monitor events that you're interesting.
     * @param looper
     *            notify on your own looper, default is main thread looper.
     */
    public HttpClientAsync(IHttpEventListener listener, Looper looper) {
        if (listener == null) {
            throw new IllegalArgumentException("listener cannot be null");
        }
        mEventListener = listener;
        mLooper = looper;
        mExecutor = new fixLooperForExecutor(looper);
        mRequestCallback = new RequestCallback();
    }

    /**
     * construct a asynchronous http request width default loopger(main looper), and the callback will run on main
     * thread(UI thread)
     */
    public HttpClientAsync(IHttpEventListener listener) {
        this(listener, Looper.getMainLooper());
    }

    /**
     * enqueue the http request in normal priority. Make sure method {@link com.uc.base.net.HttpClientAsync#getRequest(String url)} has
     * called when start send request
     */
    public void sendRequest(com.uc.base.net.IRequest request) {
        NetLog.e("shellunet", "HttpClientAsync sendRequest ");

        CreateMetrics();
        if (request instanceof com.uc.base.net.unet.adaptor.UnetRequestAdaptor) {
            mRequestAdaptor = (com.uc.base.net.unet.adaptor.UnetRequestAdaptor) request;
            if (mConnectTimeOut > 0) {
                mRequestAdaptor.setConnectTimeout(mConnectTimeOut);
            }
            if(mRequestTimeOut > 0){
                mRequestAdaptor.useSyncRequestTimeout(mRequestTimeOut);
            }
            mRequestAdaptor.start(mExecutor, mRequestCallback);
        }
    }

    @Override
    public void sendRequest(final IRequest request, final boolean highPriority) {
       sendRequest(request);
    }

    /**
     * the request will cancel and have a the cancel event callback when http request haven't start connect. the
     * connection will be close when the request have been processing.
     */
    public void cancel(IRequest request) {
        if (request == null) {
            NetLog.e("HttpClientAsync", "cancel fail!! request is null, just ignore the cancel operation");
            return;
        }
        if (request instanceof com.uc.base.net.unet.adaptor.UnetRequestAdaptor) {
            com.uc.base.net.unet.adaptor.UnetRequestAdaptor requestAdaptor = (com.uc.base.net.unet.adaptor.UnetRequestAdaptor) request;
            requestAdaptor.cancel();
        }
    }

    @Override
    public void setDataDecompressEventListener(final IDataDecompressEventListener listener) {

    }

    @Override
    public IDataDecompressEventListener getDataDecompressEventListener() {
        return null;
    }

    @Override
    public void setRequestTimeout(int timeout) {
        mRequestTimeOut = timeout;
    }

    public Looper getLooper() {
        return mLooper;
    }

    private boolean isConnectOrDnsError(int errorCode) {
        // uNet错误码没有同步到Java端，暂时这么处理
        // Ranges:
        //     0- 99 System related errors
        //   100-199 Connection related errors
        //   200-299 Certificate errors
        //   300-399 HTTP errors
        //   400-499 Cache errors
        //   500-599 ?
        //   600-699 FTP errors
        //   700-799 Certificate manager errors
        //   800-899 DNS resolver errors
        return (errorCode >= -199 && errorCode <= -100) || (errorCode >= -899 && errorCode <= -800);
    }

    // 使用backupip重试
    private boolean checkBackupIpAndRetry(Request request, int errorCode) {
        if (!AppNetworkDelegate.isNetworkConnected()) {
            return false;
        }
//        if (!isConnectOrDnsError(errorCode)) {
//            NetLog.e("shellunet", "not connect or dns error , code:" + errorCode);
//            return false;
//        }
        if (mOriginRequest == null) {
            // 记录最初的request信息
            mOriginRequest = request;
        }
        String url = mOriginRequest.getURL();
        if (url.contains("https")) {
            // https 不支持使用backupip重试
            return false;
        }
        String host = mOriginRequest.getHost();
        if (TextUtils.isEmpty(host)) {
            return false;
        }
        if (mNextBackupIpRetryIndex == 0) {
            // 第一次重试尝试获取backup ip列表
            mRetryingBackupIps = BackupIpProvider.getAddresses(host);
            NetLog.e("shellunet", "backup ips size:" + (mRetryingBackupIps == null ? 0 : mRetryingBackupIps.length));
            if (mRetryingBackupIps == null) {
                return false;
            }
        } else {
            // 非第一次重试，如果列表还有其他ip则继续，没有则退出重试
            if (mRetryingBackupIps.length <= mNextBackupIpRetryIndex) {
                NetLog.e("shellunet", "backup ip retry finished");
                return false;
            }
        }

        // 获取 backup ip 并替换url
        String ip = mRetryingBackupIps[mNextBackupIpRetryIndex];
        String newUrl = url.replace(host, ip);

        IRequest newReq = getRequest(newUrl);
        Headers.Header[] headers = mRequestAdaptor.getAllHeaders();
        if (headers != null) {
            for (Headers.Header header : headers) {
                newReq.addHeader(header.getName(), header.getValue());
            }
        }

        newReq.addHeader("Host", host);
        NetLog.e("shellunet", "Do backup ip retry:" + newUrl + " for host:" + host);
        sendRequest(newReq);

        mNextBackupIpRetryIndex ++;
        mRequestAdaptor.setUseBackupIp(true);
        return true;
    }

    private void resetBackupIpRetry() {
        mOriginRequest = null;
        mRetryingBackupIps = null;
        mNextBackupIpRetryIndex = 0;
    }

    @Override
    public void setProxy(String url, int port) {
        super.setProxy(url, port);
    }

    @Override
    public void setFollowProxy(boolean follow) {
        super.setFollowProxy(follow);
    }

    class RequestCallback extends Request.Callback {
        private ByteArrayOutputStream mByteReceived = new ByteArrayOutputStream();
        private WritableByteChannel mReceiveChannel = Channels.newChannel(mByteReceived);

        @Override
        public void onRedirectReceived(Request iRequest, Response iResponse, String s) {
            NetLog.e("shellunet", "HttpClientAsync RequestCallback onRedirectReceived ");

            if (!mEventListener.onRedirect(s)) {
                iRequest.followRedirect();
            } else {
                iRequest.cancel();
            }
        }

        @Override
        public void onResponseStarted(Request iRequest, Response iResponse) {
            NetLog.e("shellunet", "HttpClientAsync RequestCallback onResponseStarted ");

            iRequest.readNew(ByteBuffer.allocateDirect(32 * 1024));
            mEventListener.onStatusMessage(HttpUtil.getProtocolVersion(iResponse),
                    iResponse.getHttpStatusCode(), iResponse.getHttpStatusText());

            Headers shell_headers = new Headers();
            List<Map.Entry<String, String>> headers = iResponse.getAllHeadersAsList();
            for (Map.Entry<String, String> entry : headers) {
                //TODO: value should be a list.means one key should match to a list value,like set-cookies.
                //TODO: make sure getAllHeaders return normal.
                Headers.Header header = new Headers.Header(entry.getKey(), entry.getValue());
                shell_headers.addHeader(header);
            }
            mEventListener.onHeaderReceived(shell_headers);
        }

        @Override
        public void onReadCompleted(Request iRequest, Response iResponse, ByteBuffer byteBuffer) {
            byteBuffer.flip();
            try {
                mReceiveChannel.write(byteBuffer);
            } catch (IOException e) {

            }
            byteBuffer.clear();
            iRequest.readNew(byteBuffer);
        }

        @Override
        public void onSucceeded(Request iRequest, Response iResponse) {
            NetLog.e("shellunet", "#unet HttpClientAsync RequestCallback onSucceeded " + iResponse.getUrl() + " request:" + iRequest.hashCode());
            long totalLen = 0;
            if (mByteReceived != null) {
                byte[] respData = mByteReceived.toByteArray();
                totalLen = respData.length;
                mEventListener.onBodyReceived(respData, respData.length);
            }
            if (iResponse != null) {
                NotifyRequestComplete(iResponse.getUrl(), iResponse.getHttpStatusCode(), iResponse.getHttpStatusText(), iResponse.getAllHeaders());
            }
            FormatMetrics(iResponse);
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_AFTER_DECOMPRESS_BYTES, String.valueOf(totalLen));
            mEventListener.onMetrics(mMetrics);
        }

        @Override
        public void onFailed(Request iRequest, Response iResponse, RequestException e) {
            NetLog.e("shellunet", "HttpClientAsync RequestCallback onFailed " + e.toString() + " err message:" + e.getMessage() + " errorCode:" + e.getErrorCode());
            NetLog.e("shellunet", "#unet HttpClientAsync RequestCallback onFailed " + iRequest.hashCode());
            if (checkBackupIpAndRetry(iRequest, e.getErrorCode())) {
                NetLog.e("shellunet", "retry with backup ip");
                return;
            }

            mEventListener.onError(e.getErrorCode(), e.getMessage());
            NotifyRequestComplete(mRequestAdaptor.getUrl(), e.getErrorCode(), null, null);
            resetBackupIpRetry();
        }
    }
}
